@@ -1,4 +1,22 @@
 #!/usr/bin/env python
+#
+# A library that provides a Python interface to the Telegram Bot API
+# Copyright (C) 2015-2016
+# Leandro Toledo de Souza <devs@python-telegram-bot.org>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Lesser Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Lesser Public License for more details.
+#
+# You should have received a copy of the GNU Lesser Public License
+# along with this program.  If not, see [http://www.gnu.org/licenses/].
+
 
 """
 This module contains the class Updater, which tries to make creating Telegram
@@ -7,12 +25,12 @@ Bots intuitive!
 import logging
 import os
 import ssl
-from threading import Thread
+from threading import Thread, Lock
 from time import sleep
 import subprocess
 from signal import signal, SIGINT, SIGTERM, SIGABRT
 from telegram import (Bot, TelegramError, dispatcher, Dispatcher,
-                      NullHandler)
+                      NullHandler, JobQueue)
 from telegram.utils.webhookhandler import (WebhookServer, WebhookHandler)
 
 # Adjust for differences in Python versions
@@ -47,16 +65,33 @@ class Updater:
     Attributes:
 
     Args:
-        token (str): The bots token given by the @BotFather
+        token (Optional[str]): The bot's token given by the @BotFather
         base_url (Optional[str]):
         workers (Optional[int]): Amount of threads in the thread pool for
             functions decorated with @run_async
+        bot (Optional[Bot]):
+
+    Raises:
+        ValueError: If both `token` and `bot` are passed or none of them.
     """
 
-    def __init__(self, token, base_url=None, workers=4):
+    def __init__(self,
+                 token=None,
+                 base_url=None,
+                 workers=4,
+                 bot=None,
+                 job_queue_tick_interval=1.0):
+        if (token is None) and (bot is None):
+            raise ValueError('`token` or `bot` must be passed')
+        if (token is not None) and (bot is not None):
+            raise ValueError('`token` and `bot` are mutually exclusive')
 
-        self.bot = Bot(token, base_url)
+        if bot is not None:
+            self.bot = bot
+        else:
+            self.bot = Bot(token, base_url)
         self.update_queue = Queue()
+        self.job_queue = JobQueue(self.bot, job_queue_tick_interval)
         self.dispatcher = Dispatcher(self.bot, self.update_queue,
                                      workers=workers)
         self.last_update_id = 0
@@ -64,14 +99,15 @@ class Updater:
         self.running = False
         self.is_idle = False
         self.httpd = None
+        self.__lock = Lock()
 
-    def start_polling(self, poll_interval=1.0, timeout=10, network_delay=2):
+    def start_polling(self, poll_interval=0.0, timeout=10, network_delay=2):
         """
         Starts polling updates from Telegram.
 
         Args:
             poll_interval (Optional[float]): Time to wait between polling
-                updates from Telegram in seconds. Default is 1.0
+                updates from Telegram in seconds. Default is 0.0
             timeout (Optional[float]): Passed to Bot.getUpdates
             network_delay (Optional[float]): Passed to Bot.getUpdates
 
@@ -79,22 +115,25 @@ class Updater:
             Queue: The update queue that can be filled from the main thread
         """
 
-        # Create Thread objects
-        dispatcher_thread = Thread(target=self.dispatcher.start,
-                                   name="dispatcher")
-        event_handler_thread = Thread(target=self._start_polling,
-                                      name="updater",
-                                      args=(poll_interval, timeout,
-                                            network_delay))
+        with self.__lock:
+            if not self.running:
+                self.running = True
 
-        self.running = True
+                # Create Thread objects
+                dispatcher_thread = Thread(target=self.dispatcher.start,
+                                           name="dispatcher")
+                updater_thread = Thread(target=self._start_polling,
+                                        name="updater",
+                                        args=(poll_interval,
+                                              timeout,
+                                              network_delay))
 
-        # Start threads
-        dispatcher_thread.start()
-        event_handler_thread.start()
+                # Start threads
+                dispatcher_thread.start()
+                updater_thread.start()
 
-        # Return the update queue so the main thread can insert updates
-        return self.update_queue
+                # Return the update queue so the main thread can insert updates
+                return self.update_queue
 
     def start_webhook(self,
                       listen='127.0.0.1',
@@ -120,21 +159,27 @@ class Updater:
             Queue: The update queue that can be filled from the main thread
         """
 
-        # Create Thread objects
-        dispatcher_thread = Thread(target=self.dispatcher.start,
-                                   name="dispatcher")
-        event_handler_thread = Thread(target=self._start_webhook,
-                                      name="updater",
-                                      args=(listen, port, url_path, cert, key))
+        with self.__lock:
+            if not self.running:
+                self.running = True
 
-        self.running = True
+                # Create Thread objects
+                dispatcher_thread = Thread(target=self.dispatcher.start,
+                                           name="dispatcher")
+                updater_thread = Thread(target=self._start_webhook,
+                                        name="updater",
+                                        args=(listen,
+                                              port,
+                                              url_path,
+                                              cert,
+                                              key))
 
-        # Start threads
-        dispatcher_thread.start()
-        event_handler_thread.start()
+                # Start threads
+                dispatcher_thread.start()
+                updater_thread.start()
 
-        # Return the update queue so the main thread can insert updates
-        return self.update_queue
+                # Return the update queue so the main thread can insert updates
+                return self.update_queue
 
     def _start_polling(self, poll_interval, timeout, network_delay):
         """
@@ -175,9 +220,11 @@ class Updater:
             except URLError as e:
                 self.logger.error("Error while getting Updates: %s" % e)
                 # increase waiting times on subsequent errors up to 30secs
-                if current_interval < 30:
+                if current_interval == 0:
+                    current_interval = 1
+                elif current_interval < 30:
                     current_interval += current_interval / 2
-                if current_interval > 30:
+                elif current_interval > 30:
                     current_interval = 30
 
         self.logger.info('Updater thread stopped')
@@ -217,26 +264,31 @@ class Updater:
 
     def stop(self):
         """
-        Stops the polling/webhook thread and the dispatcher
+        Stops the polling/webhook thread, the dispatcher and the job queue
         """
-        self.logger.info('Stopping Updater and Dispatcher...')
-        self.logger.debug('This might take a long time if you set a high value'
-                          ' as polling timeout.')
-        self.running = False
 
-        if self.httpd:
-            self.logger.info(
-                'Waiting for current webhook connection to be closed... '
-                'Send a Telegram message to the bot to exit immediately.')
-            self.httpd.shutdown()
-            self.httpd = None
+        self.job_queue.stop()
+        with self.__lock:
+            if self.running:
+                self.running = False
+                self.logger.info('Stopping Updater and Dispatcher...')
+                self.logger.debug('This might take a long time if you set a '
+                                  'high value as polling timeout.')
 
-        self.logger.debug("Requesting Dispatcher to stop...")
-        self.dispatcher.stop()
-        while dispatcher.running_async > 0:
-            sleep(1)
+                if self.httpd:
+                    self.logger.info(
+                        'Waiting for current webhook connection to be '
+                        'closed... Send a Telegram message to the bot to exit '
+                        'immediately.')
+                    self.httpd.shutdown()
+                    self.httpd = None
 
-        self.logger.debug("Dispatcher stopped.")
+                self.logger.debug("Requesting Dispatcher to stop...")
+                self.dispatcher.stop()
+                while dispatcher.running_async > 0:
+                    sleep(1)
+
+                self.logger.debug("Dispatcher stopped.")
 
     def signal_handler(self, signum, frame):
         self.is_idle = False
@@ -244,7 +296,7 @@ class Updater:
 
     def idle(self, stop_signals=(SIGINT, SIGTERM, SIGABRT)):
         """
-        Waits for the user to press Ctrl-C and stops the updater
+        Blocks until one of the signals are received and stops the updater
 
         Args:
             stop_signals: Iterable containing signals from the signal module
